// file: driver.c
/* THis file will probably be the MAIN of the project
   version 2 of this file would take as command line arguments a file name
   that contained the time samples, sample frequency, and # points for an 
   fft.
   11/6/96  Realized that in our "real" data files, there is no signal for
      the first second.  SO we modified the FREAD section of the code to read
      about a second of the data before giving 1024 samples to the fft 
      routine.
      This code is saved as driver.cc.v2

   11/12/96  Modified so that it gets sampled data from stdin via cat.  Current
      code takes as command line arguments sample frequency and # points for
      fft. Ex: cat /u0/gray/archive/PhaseI/fs_10000/acg_1_~E6~.raw | 
               transform.exe 10000 1024
      Also set the time data threshhold for processing to be > 1000 in 
       magnitude within the first 14 time samples.  This seemed to work
       for the data of lowest frequency.
      Also implemented a scheme to not process all data.  Once we process 1024
       samples, we will disregard the next 2048 samples.  Hopefully this will 
       allow us to keep up.  2048 samples should be about 200 ms.  Nothing of 
       importance should happen for only 200 ms (I hope).
      this code will be saved as driver.cc.v3

   11/14/96  Made a few minor changes.  Changed the variable 'input' to an extern 
      called 'magnitude' and also defined the extern 'frequency' to hold the 
      frequency points determined in the fft.  also added some code in preparation
      for the sifter routine, but this should be commented out for now.
      MAJOR CHANGE:  I added code so that the first set of 1024 samples that qualify 
      will not be used.  To use a set of 1024, the previous set must also have been 
      valid.  This is to eliminate the spurious spikes we saw when doing the first
      valid set of 1024.  The variable 'processable helps control this.

   11/26/96  Made a change to print all notes and overtones--probably just for
             testing purposes
 
   11/29/96  VERSION 2.0  This version successfully creates a pipe, forks and 
      execve narecord.  Narecord writes its data to the pipe we created.  The
      parent process is basically the version one code except it does its
      reading from the pipe we created which is being written to by narecord.
      Our biggest problem was figuring out that the array of parameters that 
      are passed on the execve should begin in slot 1 because this is really an
      argv which has the executable name in slot 0.  We also had to write the 
      int_to_char function which converts the int file descriptor from the pipe
      function call to an ascii representation of that integer number so it can be
      used in the path /dev/fd/n  where n is the descriptor number.
*/

// include files
//
#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <signal.h>
#include <sys/param.h>
#include <fcntl.h>
#include <sys/file.h>
#include <string.h>
#include "note.h"

#ifdef SIGTSTP
#include <sys/file.h>
#include <sys/ioctl.h>
#endif

float frequency[1024];  /* extern variable */
float magnitude[1024]; /* extern variable */
int driver(int mode) 
{

   float fs; /* sample freq from command line */
   int i,j; /* counters */
   int N_d;
   short int samples[N];
   short int processable = 0; /* flag to know if previous samples are above threshhold */
   short int fread_count=0; /* counts fread's(used when writing to output file */
   float res; /* frequency spectrum resolution */
   NOTE_TRAIT tree[NUM_NOTES];

   float real_a[N];
   float imag_a[N];
   FILE *output_file; /* output file containing freq samples */ 
   FILE *data_file; /* for the read end of the pipe */ 
   NOTE* note_list;
   NOTE* current_note;
   extern process_number;
  

/* NEW STUFF FOR VERSION 2 */

   int data_pipe[2]; /* data_pipe[0] will be for reading and data_pipe[1] will be for writing */
   int childpid; /* for pid of narecord */
   extern char **environ;
   char *pipe_file; 

 /*  ****************** */

   void qft(int N_d, float* input, float* real_a, float* imag_a);
   void make_tree(NOTE_TRAIT*);
   NOTE* sifter(void);
   NOTE_TRAIT freq_to_name(float freq,NOTE_TRAIT* freq_note_list);
   void set_name_and_code(NOTE*, NOTE_TRAIT*);
   void set_quality(NOTE*, NOTE_TRAIT*);
   char* int_to_char(int);
   void dpl_tuned_note(NOTE*);
   void interval(NOTE*);
  
/* next lines allowed initialization of sample frequency and N point fft from 
   the command line.   We are using 10000 Hz and 1024 points, so we hard 
   coded them
      
   N_d = atoi(argv[2]);  
   fs = (float)atoi(argv[1]);
*/
 
/*  data_file was used when debuging  i.e. when we actually had to go into the 
    debugger (gdb)

   data_file = fopen("/u0/gray/archive/PhaseI/fs_10000/tru_1_~B4-.raw","r"); 
   data_file = fopen("test","w");
*/   

   N_d = 1024;
   
   fs = 10000;
   res = fs/(float)N_d;
   
   // Go ahead and build the binary search tree 
   make_tree(tree);

   for(i = 0;i<N_d;i++)
   {
      frequency[i] = i * res;
   }

/* NEW FOR V2--Set up the PIPE */

   if (pipe(data_pipe) < 0) /* data_pipe[0] will be for reading and data_pipe[1] will be for writing */
   {
      printf("Pipe error");
      exit(2);
   }
/* THAT SHOULD DO THE PIPE */

/* FORK A PROCESS */

   childpid = fork();

  /* if fork error,... */
  if( childpid < 0 )
  {
    printf("Fork error");
    exit(1);
  }

  /* if child,...narecord */
  if( childpid == 0 )
  {
    #define string0 "narecord" /* REMEMBER--0 element of argv is the executable name--specific value unimportant for execve */
    #define string1 "-s"
    #define string2 "10000" /* sample frequency for narecord */
    #define string3 "-o"
    #define string4 "left" /* channel for narecord */
    #define string5 "-t"
    #define string6 "6" /* time in seconds for narecord to record */
    #define string7 "/dev/fd/" /* the file name that the file descriptor from the pipe function call references is 
                                  /dev/fd/n where n is the ascii representation of the integer returned by the pipe call.
                                  We have to int_to_char the integer and then strcat it to this string */
   
    
    int results; /* int for execve in case it fails */
    char my_string0[128];
    char my_string1[128];
    char my_string2[128];
    char my_string3[128];
    char my_string4[128];
    char my_string5[128];
    char my_string6[128];
    char my_string7[128];
    char *command_line[9];

/* build argv for execve */

    strcpy(my_string0,string0);
    strcpy(my_string1,string1);
    strcpy(my_string2,string2);
    strcpy(my_string3,string3);
    strcpy(my_string4,string4);
    strcpy(my_string5,string5);
    strcpy(my_string6,string6);
    strcpy(my_string7,string7);

    pipe_file = int_to_char(data_pipe[1]);
    strcat(my_string7,pipe_file);

    command_line[0] = my_string0;
    command_line[1] = my_string1;
    command_line[2] = my_string2;
    command_line[3] = my_string3;
    command_line[4] = my_string4;
    command_line[5] = my_string5;
    command_line[6] = my_string6;
    command_line[7] = my_string7;
    command_line[8] = NULL;

    close(data_pipe[0]); /* Explanation of this is left as an exercise for the reader--
			    That is, the book we copied from did this (and it works).
			    The book is "UNIX Network Programming" by W. Richard Stevens p.105-106 */

    results = execve("/usr/local/netaudio/bin/narecord",command_line,environ);
    printf("Results of exec = %d\n",results);
  }

  /* if parent,...proceed almost as V1 */

  if( childpid > 0 )
  {
   close(data_pipe[1]); /* our code is reading from the pipe, so close the write end 
			 But as we stated above: Explanation of this is left as an exercise for the reader--
			 That is, the book we copied from did this (and it works).
			 The book is "UNIX Network Programming" by W. Richard Stevens p.105-106 */

   output_file = fopen("freq_dat.out","w");
   char read_pipe[56]="/dev/fd/";
   pipe_file = int_to_char(data_pipe[0]);
   strcat(read_pipe,pipe_file);
   data_file = fopen(read_pipe,"r");

/*    The next few lines will write the data we analyze back to a file in raw format.
      This is useful for debugging puposes and can be listened to.  See comment a few 
      lines down.
   
      FILE *sample_file;
      sample_file = fopen("samples.raw","wb");
  
*/
   sleep(1);
   if (mode == 1)
      printf("Data Set Processed   Note Name    Tuning Quality\n");
   while (fread(samples,sizeof(short int),N_d,data_file)==N_d)
   { /*loop until pipe doesn't have enough data to do the fft */
     
/* This is where we can write the data back to a file if so desired (to listne to, or to use in debugging 
     
      fwrite(samples,sizeof(short int),N_d,sample_file);
*/

      fread_count++;
      for(j=0;j<14;j++)
      { /* if a note is being played, then within 10 samples should have magnitude > 1000 */
         if (abs(samples[j])>1000)
         { /* if magnitude not greater than 1000 then nobody's playing? */
            if (processable)  /* when coming from silence, process the 2nd good set of samples */
            {
            // build function to get the QFT of
	       process_number++;
               for(i=0;i<N_d;i++) 
	          magnitude[i]=(float)samples[i];
	       qft(N_d, magnitude, real_a, imag_a);
               fprintf(output_file,"the results are for fread number %d\n",fread_count);
	       for (i=0; i<N_d; i++) 
 	       {
	          magnitude[i] = sqrt(real_a[i]*real_a[i]+imag_a[i]*imag_a[i]);

	       } /* end for */

               /* here we have frequency and magnitude--do something with it */
               note_list = sifter();  /* determine from the spectrum, what frequencies should be notes*/
               set_name_and_code(note_list,tree); /* after this, all notes should be named and have code set */
               set_quality(note_list,tree); /* the big test--can we classify as sharp, in tune, or flat */
               
               if (mode==1)
                     dpl_tuned_note(note_list);
               if ((mode==2)&&(process_number==3))
                     interval(note_list);

               /* let's write everything out to the file and see what we get */
	       current_note = note_list;
               while (current_note != NULL)
	       {
		  fprintf(output_file,"freq              ");
		  for(i=0;i<12;i++)
	             fprintf(output_file,"%-#7.2f   ",current_note[i].freq);
		  fprintf(output_file,"\nmagnitude\t");
                  for(i=0;i<12;i++)
	             fprintf(output_file,"  %1.2e",current_note[i].magnitude);
		  fprintf(output_file,"\novertone          ");
                  for(i=0;i<12;i++)
	             fprintf(output_file,"%d         ",current_note[i].overtone);
		  fprintf(output_file,"\nname              ");
                  for(i=0;i<12;i++)
	             fprintf(output_file,"%s  ",current_note[i].name);
		  fprintf(output_file,"\ncode              ");
                  for(i=0;i<12;i++)
	             fprintf(output_file,"%d         ",current_note[i].code);
		  fprintf(output_file,"\nquality           ");
                  for(i=0;i<12;i++)
	             fprintf(output_file,"%+d        ",current_note[i].quality);
		  fprintf(output_file,"\n\n");

                  current_note = current_note[0].next_note;

               } /* end while */
                  
               /* throw away some data to hopefully keep up in real time */
               i=0;
               while ((i<2)&&(fread(samples,sizeof(short int),N_d,data_file)==N_d))
               { /* throw away a few 100 mil seconds in order to keep up */
                  i++;
                  fread_count++;
               } /* end while */
               break; /* break the for loop because we processed */
	    } /* end if */   
            else
               processable = TRUE;  /* this is the first processable data (last was silence) */
               break;
	 } /* end if */
         if (j==13)
            processable = FALSE; /* the first 14 samples are < threshhold--we must have silence */
      } /* end for */
   } /* end while */
   fclose(output_file);
}
   return 0;
}
