/* file: $PDSPurier_transform/v1.1/ft_qv1.1/ft_0.cc */
/* */

/* include files */
/* */
#include "ft_qft_0.h"


/*	QFT constants	 */
/* */
#define	sec0	0.5000000000000000f
#define	sec1	0.5411961001461970f
#define	sec2	0.7071067811865475f
#define	sec3	1.3065629648763763f

#define	sqrt2	sec2

#define	pi		((float) 3.141592653589793)

/* QFT externals	 */
/* */
static	float	**ws;   /*	Workspace pointers */
static	float	*sc;	/*	Secant table addr.	 */
static	int	*nc;	/*	Current QFT length	 */
static	int	*ic;	/*	Current secant inc.	 */
static	int	*mc;	/*	Current output pruning	 */
static	int	nn;	/*	Pruned QFT input length */
static	int	mm;	/*	Pruned QFT output length */
static	int	ii;	/*	Current QFT recursion level */

#define Debug



int qft(int N_d, float* data_a, float* real_coeff_a, float* imag_coeff_a){
  void qft_init_cc(int N_d);
  void qft_close_cc();
  void real_dct_cc(float *coeff_a, float *x);
  void real_dst_cc(float *coeff_a, float *x);
  int j;
  float  *xx, *y;
  int	i, n2;
  /* initialize memory for qft computation */
  /* */
  
  qft_init_cc(N_d);

  /* Set up constants	 */
  /* */
  if (N_d < MM)
      nn = MM;
  else
      nn = N_d;
  
  mm = N_d;
  n2 = N_d>>1;
  xx = ws[ii = 0];
  
  
  for (i = 1; i < mc[0]; i = i+1)
      mc[i] = N_d;
  
      /* Form even function */
      /* */
      if (nn <= n2) {
	  y = data_a;
      }
      
      else {
	  xx[0] = data_a[0];
	  
	  for (i = 1; i <= N_d-nn; i = i+2){
	      xx[i] = data_a[i];
	      xx[i+1] = data_a[i+1];
	  }
	  
	  xx[i] = (data_a[i]+data_a[N_d-i]);
	  i = i+1;
	  
	  for (;i < n2; i = i+2) {
	      xx[i] = (data_a[i]+data_a[N_d-i]);
	      xx[i+1] = data_a[(i+1)]+data_a[N_d-(i+1)];
	  }
	  
	  xx[i] = data_a[i];
	  i = i+1;
	  
	  for (;i < MM; i = (i+1))
	      xx[i] = 0.0;
	  y = xx;
      }
      
      /* Do a real DCT */
      /* */
      real_dct_cc(real_coeff_a, y);


      /*    Form negative odd function */
      /* */
    if (nn <= n2) {
      xx[1] = -data_a[1];
      
      for (i = 2; i < nn; i = i+2) {
	  xx[i] = -data_a[i];
	  xx[i+1] = -data_a[i+1];
      }
    }
    else {
      
      for (i = 1; i <= N_d-nn; i = i+2) {
	xx[i] = -data_a[i];
	xx[i+1] = -data_a[i+1];
      }

      
      xx[i] = data_a[N_d-i]-data_a[i];
      i = i+1;
      
      for (;i < n2; i = i+2) {
	xx[i] = data_a[N_d-i]-data_a[i];
	xx[i+1] = data_a[N_d-(i+1)]-data_a[i+1];
      }
    }
    
    for (; i < MM; i = i+1)
      xx[i] = 0.0;
    
    /* Do a real DST */
    /* */
    if (imag_coeff_a == NULL){
      real_dst_cc(&real_coeff_a[n2+1], &xx[1]);
    }

    else {
      real_dst_cc(&imag_coeff_a[1], &xx[1]);
      imag_coeff_a[0] = imag_coeff_a[n2] = 0.0;
    }


    /* create symmetric part of the spectrum */
    /* */
    for (j = 1;j <= N_d/2;j++){
      imag_coeff_a[N_d - j] = -imag_coeff_a[j];
      real_coeff_a[N_d - j] = real_coeff_a[j];
    }

    imag_coeff_a[N_d/2] = imag_coeff_a[0] = 0.0;
    
      
  /* deallocate memory */
  /* */
  qft_close_cc();
  
  /* exit gracefully */
  /* */
  return 1;
    
}

  

/*  Program: qftinit, Quick Fourier Transform	 */
/*  secant table and workspace initializer.		 */

int qft_init_cc(int N_d){
  
  int	n2, i, m;
  float q;
  /* Compute some constants	 */
  /* */
  m = 0;
  i = N_d;
  while(i = i>>1)
    m = m+1;
  
  n2 = N_d>>1;
  
  /* Allocate the QFT memory	 */
  /* */
  if ((ws = (float **) malloc(sizeof(float)*m + 
			      sizeof(int)*3*m +
			      sizeof(float)*(3*(N_d + m)))) == NULL) {
    printf("\n\a Out of memory in qftinit().\n");
    return 0;
  }
  
  nc = (int *)&ws[m];
  ic = &nc[m];
  mc = &ic[m];
  sc = (float *)&mc[m];
  ws[0] = &sc[n2-1];
  
  /* Initialize tables	 */
  /* */
  nc[0] = N_d;
  ic[0] = 1;
  mc[0] = m;
  
  for (i = 1; i < m; i = i+1) {
    nc[i] = nc[i-1]>> 1;
    ic[i] = ic[i-1]<<1;
    
    if (i > 1)
      ws[i] = ws[i-1]+(2*(nc[i-1]+2));
    else
      ws[i] = ((ws[i-1]+nc[i])+1);	
  }	
  
  /* Compute scaled secants	 */
  /* */
  q = pi / N_d;
  
  for (i = 0; i < n2-1; i = i+1)
    sc[i] = 0.5 / (float)cos((q)*(float)i);	
  
  /* exit gracefully */
  /* */
  return 1;
}

int qft_close_cc(){
  
  /* Deallocate memory space */
  /* */
  free(ws);
  
  /* exit gracefully */
  /* */
  return 1;
}


int real_dct_cc(float *coeff_a, float *x){

	float	*XX, *xx, t0, t1, t2, t3, t4;
	long	i, j, k, n, nq, n2, n2p1;

/*	Increment level	& start	*/
	n = nc[++ii];
	xx = ws[ii];

/*	Last recursion level ?	*/

	if (n == N2)
	{
	/*	Form even function	*/

	 	xx[0] = x[0] + x[8];
		xx[1] = x[1] + x[7];
		xx[2] = x[2] + x[6];
		xx[3] = x[3] + x[5];

	/*	Save even DCT coefficients	*/

		t0 = xx[0] - x[4];
		t3 = xx[0] + x[4];
		t2 = xx[1] + xx[3];
		t1 = (xx[1] - xx[3])*sqrt2;
		t4 = t3 + xx[2];

		coeff_a[0] = t4 + t2;
		coeff_a[2] = t0 + t1;
		coeff_a[4] = t3 - xx[2];
		coeff_a[6] = t0 - t1;
		coeff_a[8] = t4 - t2;
	
	/*	Form twiddled odd function	*/

	 	xx[0] = (x[0] - x[8])*sec0;
		xx[1] = (x[1] - x[7])*sec1;
		xx[2] = (x[2] - x[6])*sec2;
		xx[3] = (x[3] - x[5])*sec3;

	/*	Save odd DCT coefficients	*/

		t1 = (xx[1] - xx[3])*sqrt2;	
		t2 = xx[1] + xx[3];
		t4 = xx[0] + xx[2];

		coeff_a[1] = t4 + t2 + (t0 = xx[0] + t1);
		coeff_a[3] = t0 + (t3 = xx[0] - xx[2]);
		coeff_a[5] = t3 + (t1 = xx[0] - t1);
		coeff_a[7] = t1 + t4 - t2; 
	}
	else
	{
		n2 = n >> 1;
		n2p1 = n2 + 1;
		nq = ic[ii];  

		if ((k = mc[ii]) > n2)
			k = n2;

	/*	Form even-odd functions	*/

		if (nn <= n2p1)
			for (i = j = 0; i < nn; i++, j += nq)
			    xx[i+n2p1] = (xx[i] = x[i])*sc[j];
		else
		    {			
			for (i = j = 0; i <= n-nn; i++, j += nq)
			    xx[i+n2p1] = (xx[i] = x[i])*sc[j];

			for (; i < n2; i++, j+= nq)
			{
				xx[i] = x[i] + x[n-i];
				xx[i+n2p1] = (x[i] - x[n-i])*sc[j];
			}

			xx[i] = x[i];
			xx[i+n2p1] = 0.0;
		}

	/*	Do recursive DCTs	*/
		
		    real_dct_cc((XX = &xx[n+2]), xx);
		    real_dct_cc(&XX[n2p1], &xx[n2p1]);
		
	/*	Save DCT coefficients	*/

		t0 = XX[n2p1];

		for (i = 0; i < k; i += 2)
		{	
			coeff_a[j = i+i] = XX[i];
			coeff_a[j+1] = t0 + (t1 = XX[i+n2p1+1]);
			coeff_a[j+2] = XX[i+1];
			coeff_a[j+3] = t1 + (t0 = XX[i+n2p1+2]);
		}

		coeff_a[i+i] = XX[i];
	}

/*	Decrement level & exit	*/

	ii--;
	return 1;

}



int real_dst_cc(float* X, float* x){


	float	*XX, *xx, t0, t1, t2;
	long	i, j, k, n, nq, n2, n2m1;

/*	Increment level & start	*/

	n = nc[++ii];
	xx = ws[ii];
	
/*	Last recursion level ?	*/

	if (n == N2)
	{
	/*	Form odd function	*/
				
		xx[0] = x[0] - x[6];
		xx[1] = x[1] - x[5];
		xx[2] = x[2] - x[4];

	/*	Save odd DST coefficients	*/

		t0 = (xx[0] + xx[2])*sqrt2;

		X[1] = t0 + xx[1];
		X[3] = xx[0] - xx[2];
		X[5] = t0 - xx[1];
	
	/*	Form twiddled even function	*/

		xx[0] = (x[0] + x[6])*sec1;
		xx[1] = (x[1] + x[5])*sec2;
		xx[2] = (x[2] + x[4])*sec3;

	/*	Save even DST coefficients	*/

		t0 = (xx[0] + xx[2])*sqrt2;
	
  		X[0] = (t1 = t0 + xx[1]) + x[3];
		X[2] = (t2 = xx[0] - xx[2]) + t1 - x[3];
		X[4] = (t0 = t0 - xx[1]) + t2 + x[3];
		X[6] = t0 - x[3];
	}
	else
	{
		n2 = n >> 1;
		n2m1 = n2 - 1;
		nq = ic[ii];

		if ((k = mc[ii]) > n2m1)
			k = n2m1;

	/*	Form even-odd functions	*/

		if (nn <= n2)
			for (i = 0, j = nq; i < nn-1; i++, j += nq)
				xx[i+n2m1] = (xx[i] = x[i])*sc[j];
		else
		{				
			for (i = 0, j = nq; i < n-nn; i++, j += nq)
				xx[i+n2m1] = (xx[i] = x[i])*sc[j];
				
			for (;i < n2m1; i++, j += nq)
			{
				xx[i] = x[i] - x[n-2-i];
				xx[i+n2m1] = (x[i] + x[n-2-i])*sc[j];
			}
		}
	
	/*	Do recursive DSTs	*/

		    real_dst_cc((XX = &xx[n-2]), xx);	
		    real_dst_cc(&XX[n2m1], &xx[n2m1]);

	/*	Save DST coefficients	*/
		
		X[1] = XX[0];

		if (n2 >= nn)
		{
			t0 = X[0] = XX[n2m1];
 
			for (i = 1; i < k; i += 2)
			{
	 			X[j = i+i] = (t1 = XX[i+n2m1]) + t0;
				X[j+1] = XX[i];
	 			X[j+2] = (t0 = XX[i+n2m1+1]) + t1;
				X[j+3] = XX[i+1];
			}

			X[i+i] = t0;
		}
		else
		{
			X[0] = (t1 = XX[n2m1]) + (t0 = x[n2m1]);

			for (i = 1; i < k; i += 2)
			{
	 			X[j = i+i] = (t2 = XX[i+n2m1]) + t1 - t0;
				X[j+1] = XX[i];
	 			X[j+2] = (t1 = XX[i+n2m1+1]) + t2 + t0;
				X[j+3] = XX[i+1];
			}

			X[i+i] = t1 - t0;
		}
	}

/*	Decrement level	& exit	*/

	ii--;	
	return 1;
  
  
}






