/* First cut--Pass in the pointer to note_list and the head of the 
   tree set the quality of the note based on the quality of the 
   highest overtone for all notes in the note list 

   11.27.96  Realized that I was setting the quality field in 
      current_note[index] instead of current_note[0].  Fixed it.
*/

#include "note.h"
#include <stdlib.h>

void set_quality(NOTE* note_list, NOTE_TRAIT* tree)
{
   int index;
   int int_tolerance=0;
   float numerator,
         denominator,
         tolerance;
   NOTE* current_note;
   NOTE_TRAIT tuning_note;
   NOTE_TRAIT freq_to_name(float, NOTE_TRAIT*);
   void correct(float, float*);

   current_note = note_list; 
   do
   {
      if ((current_note[11].freq != 0)&&(current_note[11].freq <= MAX))
      {
         index = 11;
         correct(CORRECT_11,&(current_note[index].freq));
         tuning_note = freq_to_name(current_note[index].freq,tree);
      }
      else if ((current_note[10].freq != 0)&&(current_note[10].freq <= CORRECTED_MAX))
      {
         index = 10;
         correct(CORRECT_10,&(current_note[index].freq));
         tuning_note = freq_to_name(current_note[index].freq,tree);
      }
      else if ((current_note[9].freq != 0)&&(current_note[9].freq <= CORRECTED_MAX))
      {
         index = 9;
         correct(CORRECT_09,&(current_note[index].freq));
         tuning_note = freq_to_name(current_note[index].freq,tree);
      }
      else if ((current_note[8].freq != 0)&&(current_note[8].freq <= MAX))
      {
         index = 8;
         correct(CORRECT_08,&(current_note[index].freq));
         tuning_note = freq_to_name(current_note[index].freq,tree);
      }
      else if ((current_note[7].freq != 0)&&(current_note[7].freq <= MAX))
      {
         index = 7;
         tuning_note = freq_to_name(current_note[index].freq,tree);
      } 
      else if ((current_note[6].freq != 0)&&(current_note[6].freq <= CORRECTED_MAX))
      {
         index = 6;
         correct(CORRECT_06,&(current_note[index].freq));
         tuning_note = freq_to_name(current_note[index].freq,tree);
      }
      else if ((current_note[5].freq != 0)&&(current_note[5].freq <= MAX))
      {
         index = 5;
         correct(CORRECT_05,&(current_note[index].freq));
         tuning_note = freq_to_name(current_note[index].freq,tree);
      }
      else if ((current_note[4].freq != 0)&&(current_note[4].freq <= CORRECTED_MAX))
      {
         index = 4;
         correct(CORRECT_04,&(current_note[index].freq));
         tuning_note = freq_to_name(current_note[index].freq,tree);
      }
      else if ((current_note[3].freq != 0)&&(current_note[3].freq <= MAX))
      {
         index = 3;
         tuning_note = freq_to_name(current_note[index].freq,tree);
      }
      else if ((current_note[2].freq != 0)&&(current_note[2].freq <= MAX))
      {
         index = 2;
         correct(CORRECT_02,&(current_note[index].freq));
         tuning_note = freq_to_name(current_note[index].freq,tree);
      }
      else if ((current_note[1].freq != 0)&&(current_note[1].freq <= MAX))
      {
         index = 1;
         tuning_note = freq_to_name(current_note[index].freq,tree);
      }
      else 
      {
         index = 0;
         tuning_note = freq_to_name(current_note[index].freq,tree);
      }
      numerator = current_note[index].freq - tuning_note.freq;
      if (numerator >= 0)
      {
         denominator = (FACTOR*tuning_note.freq); /*FACTOR is 1.059-1 */
         tolerance = numerator/denominator;
	 int_tolerance=(int)(tolerance*100);

         if (int_tolerance <= IN_TUNE_TOLERANCE)
            current_note[0].quality = IN_TUNE;
         else 
            current_note[0].quality = int_tolerance;
      }
      else /* numerator was negative i.e. we are flat */
      {
         denominator = (tuning_note.freq)*FACTOR_INV; /* find distance between note and previous note */
         tolerance = numerator/denominator; /* should be a negative number */
         int_tolerance=(int)(tolerance*100); 

	 if (int_tolerance >=(-1* IN_TUNE_TOLERANCE))
            current_note[0].quality = IN_TUNE;
         else 
            current_note[0].quality = int_tolerance;
      }
      current_note = current_note[0].next_note; /* find the quality of the next note in note_list */
   }while (current_note != NULL);
}



