/***************************************************************************/
/*		       							   */
/*	Written by: David E. Gray, Sept. 1996				   */
/*		       							   */
/*	This is a function designed to generate a signal composed purely   */
/*	of sinusoids.  It expects 4 variables to be passed into it:	   */
/*		fs: the frequency at which the signal is sampled           */
/*		    (in Hertz).						   */
/*		waves: a pointer to type SINUSOID. 			   */
/*		       SINUSOID is defined in struct.h as: 		   */
/*									   */
/*			   typedef struct wave{				   */
/*			      float A;					   */
/*			      long int freq;				   */
/*			      float phi;				   */
/*			      struct wave *next_wave;			   */	
/*			   }SINUSOID;					   */
/*		       							   */
/*			This pointer is a head of a list of sinusoids that */
/*			are present in the sample.			   */
/*		       							   */
/*			Each wave keeps track of A, freq, and phi in the   */
/*			following equation:				   */ 
/*		       							   */
/*                         y = A*sin(freq*t + phi)			   */
/*		       							   */
/*		sample: the length of the sample in seconds		   */
/*		num: the number of samples in the list (for scaling 	   */
/*		     purposes)						   */
/*		       							   */
/*	The function will write the data to the file sin.raw.  Each sample */
/*	will be a 16-bit signed integer between +/- 32767.		   */
/*		       							   */
/***************************************************************************/

#include <stdio.h>
#include <math.h>
#include "struct.h"

void sine_wave(long int fs, SINUSOID *waves, float sample, int num) {

   long int n;                  /*sample number*/
   double y;                    /*output value*/
   short int out[1];		/*output value in int format*/
   SINUSOID *current;		/*current sin wave*/
   FILE *fout;
   /* pi is defined in math.h as M_PI=3.14159265358979323846 */

   fout=fopen("sin.raw","wb");	/*open file as a writable binary file*/
 
   n=0;                         /* initialize counter */
   while(((float)n/fs)<=sample){
      y=0;
      current=waves;		/* start at beginning of list */
      while(current!=NULL){
         y=y + current->A*sin(2.0*M_PI*(float)current->freq*(float)n/fs + current->phi);
         current=current->next_wave; /*move to next wave*/
      }
      y=32767.0/(float)num*y;
      out[0]=(short int)y;
      fwrite(out,sizeof(short int),1,fout);   /* (y) data for listening to */
/*      fprintf(fout,"%d\n",(int)y);   /* (y) data for listening to */
/*      fprintf(fout,"%ld %d\n",n,(int)y); /* (x,y) data for graphing */
      n++;
   }
   fclose(fout);

   return;
} 
